<?php


class PowerPressWebSub
{
    const DEFAULT_HUB = "https://pubsubhubbub.appspot.com";
    private $hub;
    private $wp_remote_options = array(
        "user-agent" => 'Blubrry PowerPress/'.POWERPRESS_VERSION,
        "httpversion" => '1.1',
        "headers" => array('Content-Type' => 'application/x-www-form-urlencoded'),
        "body" => array(
            'hub.mode' => 'publish'
        )
    );

    /**
     * Websub constructor.
     * @param string $hub URL to the hub to use, defaults to <code>$DEFAULT_HUB</code>
     */
    public function __construct($hub = self::DEFAULT_HUB){
        $this->hub = $hub;
    }

    /**
     * Publishes update to the hub
     * @param string $feedUrl URL of feed to update
     * @throws Exception Throws exception if HTTP response is not 204 or 202 and returns HTTP body
     */
    public function publish($feedUrl) {
        if($this->isWebsubEnabled()) {
            $this->wp_remote_options['body']['hub.url'] = $feedUrl;
            $response = wp_remote_post($this->hub, $this->wp_remote_options);

            if (is_wp_error($response)) // Handle system level errors
            {
                if ($response->get_error_message())
                    throw new Exception($response->get_error_message());
                else
                    throw new Exception("HTTP error response " . $response->get_error_code());
                return false;
            }

            if ($response['response']['code'] > 299) {
                if (!empty($response['body'])) // Handle service level errors
                    throw new Exception($response['body']);
                else
                    throw new Exception("HTTP response " . $response['response']['code']);
                return false;
            }

            return true;
        }
    }

    /**
     * @return string
     */
    public function getHub()
    {
        return $this->hub;
    }

    public function isWebsubEnabled() {
        require_once 'powerpressadmin.php';
        $GeneralSettings = powerpress_get_settings('powerpress_general');
        if(!empty($GeneralSettings['websub_enabled']) && $GeneralSettings['websub_enabled'] == 1) {
            return true;
        }
        else {
           return false;
        }
    }
}